Exception: SSE::Errors::StreamClosedByServerError
- Inherits:
-
StandardError
- Object
- StandardError
- SSE::Errors::StreamClosedByServerError
- Defined in:
- lib/ld-eventsource/errors.rb
Overview
An exception class indicating that the server ended the stream without an error. This is passed to the error handler specified in Client#on_error, and the client then reconnects.
Instance Method Summary collapse
-
#initialize ⇒ StreamClosedByServerError
constructor
A new instance of StreamClosedByServerError.
Constructor Details
#initialize ⇒ StreamClosedByServerError
Returns a new instance of StreamClosedByServerError.
88 89 90 |
# File 'lib/ld-eventsource/errors.rb', line 88 def initialize super("stream closed by server") end |